# -*- coding: utf-8 -*-
# @Time    : 2020/12/20 09:44:54 
# @Author  : ddvv
# @Site    : https://ddvvmmzz.github.io
# @File    : __init__.py 
# @Software: Visual Studio Code


from . import eml, html, zip, ole, rtf, other, lnk
